@@ -30,13 +30,13 @@
 
 /* Python object containing data baked from a (struct rtnl_addr) */
 typedef struct PyNetlinkIPaddress {
-	PyObject_HEAD
-        int family;                     /**< int: must be AF_INET or AF_INET6 */
-	PyObject *local;		/**< string: Configured local IP address */
-	PyObject *peer;		        /**< string: Configured peer IP address */
-	PyObject *ipv4_broadcast;	/**< string: Configured IPv4 broadcast address */
-	int prefixlen;		        /**< int: Configured network prefix (netmask) */
-        PyObject *scope;                /**< string: IP address scope */
+    PyObject_HEAD
+    int family;  /**< int: must be AF_INET or AF_INET6 */
+    PyObject *local;  /**< string: Configured local IP address */
+    PyObject *peer;  /**< string: Configured peer IP address */
+    PyObject *ipv4_broadcast;  /**< string: Configured IPv4 broadcast add. */
+    int prefixlen;  /**< int: Configured network prefix (netmask) */
+    PyObject *scope;  /**< string: IP address scope */
 } PyNetlinkIPaddress;
 extern PyTypeObject ethtool_netlink_ip_address_Type;
 
@@ -45,11 +45,11 @@ extern PyTypeObject ethtool_netlink_ip_address_Type;
  *
  */
 typedef struct {
-	PyObject_HEAD
-	PyObject *device;                   /**< Device name */
-	int index;                          /**< NETLINK index reference */
-	PyObject *hwaddress;                /**< string: HW address / MAC address of device */
-	unsigned short nlc_active;	    /**< Is this instance using NETLINK? */
+    PyObject_HEAD
+    PyObject *device;  /**< Device name */
+    int index;  /**< NETLINK index reference */
+    PyObject *hwaddress;  /**< string: HW address / MAC address of device */
+    unsigned short nlc_active;  /**< Is this instance using NETLINK? */
 } PyEtherInfo;
 
 
