@@ -25,6 +25,7 @@
 #   net-tools-1.60/lib/interface.c
 # within ife_print_long()
 
+import os
 import re
 from subprocess import Popen, PIPE
 import unittest
@@ -55,23 +56,30 @@ group_ip4addr = ('(' + dec + dot +
                  dec + dot +
                  dec + ')')
 
+
 def parse_ip4addr(addr):
     import socket
     return (socket.inet_aton(addr))
 
+
 class IfConfig:
     """
     Wrapper around a single invocation of "ifconfig"
     """
+
     def __init__(self, stdout=None, debug=False):
         if stdout is not None:
             self.stdout = stdout
             self.stderr = ''
         else:
-            p = Popen('ifconfig', stdout=PIPE, stderr=PIPE)
+            env = os.environ.copy()
+            env.update(LANG='C.utf8')
+            p = Popen('ifconfig', stdout=PIPE, stderr=PIPE,
+                      env=env, universal_newlines=True)
             self.stdout, self.stderr = p.communicate()
             if self.stderr != '':
-                raise ValueError('stderr from ifconfig was nonempty:\n%s' % self.stderr)
+                raise ValueError(
+                    'stderr from ifconfig was nonempty:\n%s' % self.stderr)
         if 0:
             print('self.stdout: %r' % self.stdout)
         self.devices = []
@@ -105,6 +113,11 @@ class IfConfig:
                 curdev._parse_rest_of_first_line_old(mo.group(2))
                 continue
 
+            # If we don't have current device yet, doesn't make sense to
+            # read the rest of output lines
+            if curdev is None:
+                continue
+
             if self.oldFormat:
                 curdev._parse_line_old(line)
             else:
@@ -116,10 +129,12 @@ class IfConfig:
                 return dev
         raise ValueError('device not found: %r' % devname)
 
+
 class Device:
     """
     Wrapper around a device entry within the output of "ifconfig"
     """
+
     def __init__(self, name, debug=False):
         self.name = name
         self.debug = debug
@@ -175,19 +190,22 @@ class Device:
     def get_netmask_bits(self):
         # Convert a dotted netmask string to a bitcount int
         # e.g. from "255.255.252.0" to 22:
+        if not self.netmask:
+            return 0
         packed = parse_ip4addr(self.netmask)
         # count bits in "packed":
         result = 0
         for ch in packed:
-            ch = ord(ch)
+            ch = ord(ch) if isinstance(ch, str) else ch
             while ch:
                 if ch & 1:
                     result += 1
-                ch /= 2
+                ch //= 2
         return result
 
     def __repr__(self):
-        return ('Device(name=%(name)r, flagsint=%(flagsint)r, flagsstr=%(flagsstr)r, mtu=%(mtu)r)'
+        return ('Device(name=%(name)r, flagsint=%(flagsint)r, '
+                'flagsstr=%(flagsstr)r, mtu=%(mtu)r)'
                 % (self.__dict__))
 
     def _debug(self, groups):
@@ -197,7 +215,7 @@ class Device:
     def _parse_rest_of_first_line(self, text):
         m = re.match(r'flags=([0-9]+)<(.*)>  mtu ([0-9]+)(.*)', text)
         if not m:
-            raise ValueError('unable to parse: %r'% text)
+            raise ValueError('unable to parse: %r' % text)
         self._debug(m.groups())
         self.flagsint = int(m.group(1))
         self.flagsstr = m.group(2)
@@ -216,7 +234,7 @@ class Device:
     def _parse_rest_of_first_line_old(self, text):
         m = re.match('Link encap:(\w+ ?\w*)\s*(HWaddr )?(\S*)', text)
         if not m:
-            raise ValueError('unable to parse: %r'% text)
+            raise ValueError('unable to parse: %r' % text)
         self.hwtitle = m.group(1).strip()
         if m.group(2):
             self.hwaddr = m.group(3)
@@ -245,7 +263,7 @@ class Device:
         m = re.match(ws
                      + 'inet6 ' + group_nonws + ws
                      + 'prefixlen' + ws + group_dec + ws
-                     + 'scopeid '+ group_nonws,
+                     + 'scopeid ' + group_nonws,
                      line)
         if m:
             self._debug(m.groups())
@@ -311,7 +329,7 @@ class Device:
 
         # e.g. '        TX packets 2949524  bytes 344092625 (328.1 MiB)'
         m = re.match(ws
-                     + 'TX packets '+ group_dec + ws
+                     + 'TX packets ' + group_dec + ws
                      + 'bytes ' + group_dec + ws
                      + '\(.*\)',
                      line)
@@ -321,7 +339,8 @@ class Device:
             self.txbytes = int(m.group(2))
             return
 
-        # e.g. '        TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0'
+        # e.g. '        TX errors 0  dropped 0 overruns 0  carrier 0
+        # collisions 0'
         m = re.match(ws
                      + 'TX errors ' + group_dec + ws
                      + 'dropped ' + group_dec + ws
@@ -357,7 +376,8 @@ class Device:
                 self.baseaddr = int(m.group(1), 16)
                 line = m.group(2)
 
-            m = re.match('memory ' + group_o_hex + '-' + group_hex + ws + '(.*)',
+            m = re.match('memory ' + group_o_hex + '-' +
+                         group_hex + ws + '(.*)',
                          line)
             if m:
                 self.memstart = int(m.group(1), 16)
@@ -401,7 +421,7 @@ class Device:
         m = re.match(ws
                      + 'inet6 addr: ' + group_nonws + '/'
                      + group_dec + ws
-                     + 'Scope:'+ group_nonws,
+                     + 'Scope:' + group_nonws,
                      line)
         if m:
             self._debug(m.groups())
@@ -410,7 +430,7 @@ class Device:
             self.scopeid = m.group(3)
             return
 
-        #UP BROADCAST RUNNING MULTICAST  MTU:1500  Metric:1
+        # UP BROADCAST RUNNING MULTICAST  MTU:1500  Metric:1
         m = re.match(ws + '(.*)'
                      + '  MTU:' + group_dec + ws
                      + 'Metric:' + group_dec
@@ -424,7 +444,7 @@ class Device:
             # It might have "  Outfill:%d  Keepalive:%d":
             m = re.match(ws
                          + 'Outfill:' + group_dec + ws
-                         + 'Keepalive:'+ group_dec,
+                         + 'Keepalive:' + group_dec,
                          m.group(4))
             if m:
                 self._debug(m.groups())
@@ -432,7 +452,7 @@ class Device:
                 self.keepalive = int(m.group(2))
             return
 
-        #RX packets:4458926 errors:0 dropped:0 overruns:0 frame:0
+        # RX packets:4458926 errors:0 dropped:0 overruns:0 frame:0
         m = re.match(ws
                      + 'RX packets:' + group_dec + ws
                      + 'errors:' + group_dec + ws
@@ -449,13 +469,13 @@ class Device:
             self.rxframe = int(m.group(5))
             return
 
-        #"             compressed:%lu\n"
+        # "             compressed:%lu\n"
         m = re.match(ws + 'compressed:' + group_dec, line)
         if m:
             self.rxcompressed = int(m.group(1))
             return
 
-        #TX packets:3536982 errors:0 dropped:0 overruns:0 carrier:0
+        # TX packets:3536982 errors:0 dropped:0 overruns:0 carrier:0
         m = re.match(ws
                      + 'TX packets:' + group_dec + ws
                      + 'errors:' + group_dec + ws
@@ -472,7 +492,7 @@ class Device:
             self.txcarrier = int(m.group(5))
             return
 
-        #"          collisions:%lu compressed:%lu txqueuelen:%d "
+        # "          collisions:%lu compressed:%lu txqueuelen:%d "
         m = re.match(ws + 'collisions:' + group_dec + ws + '(.*)', line)
         if m:
             self._debug(m.groups())
@@ -488,7 +508,7 @@ class Device:
                 self.txqueuelen = int(m.group(1))
             return
 
-        #RX bytes:3380060233 (3.1 GiB)  TX bytes:713438255 (680.3 MiB)
+        # RX bytes:3380060233 (3.1 GiB)  TX bytes:713438255 (680.3 MiB)
         m = re.match(ws + 'RX bytes:' + group_dec + ws + '\(.*\)' + ws
                      + 'TX bytes:' + group_dec + ws + '\(.*\)',
                      line)
@@ -498,7 +518,7 @@ class Device:
             self.txbytes = int(m.group(2))
             return
 
-        #Interrupt:17 Memory:da000000-da012800
+        # Interrupt:17 Memory:da000000-da012800
         m = re.match(ws + 'Interrupt:' + group_dec + '\s*(.*)', line)
         if m:
             self._debug(m.groups())
@@ -525,12 +545,11 @@ class Device:
 
         return
 
-        raise ValueError('parser could not handle line: %r' % line)
-
-#ifconfig = IfConfig()
-#for dev in ifconfig.devices:
+# ifconfig = IfConfig()
+# for dev in ifconfig.devices:
 #    print(dev)
 
+
 class ParserTests(unittest.TestCase):
     def test_full(self):
         # Parse a complex pre-canned output from ifconfig
@@ -590,11 +609,11 @@ vnet1: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500
         self.assertEqual(eth1.flagsint, 4163)
         self.assertEqual(eth1.flagsstr, 'UP,BROADCAST,RUNNING,MULTICAST')
         self.assertEqual(eth1.mtu, 1500)
-        #        inet 1.12.123.124  netmask 255.255.252.0  broadcast 1.12.123.255
+        # inet 1.12.123.124  netmask 255.255.252.0  broadcast 1.12.123.255
         self.assertEqual(eth1.inet, '1.12.123.124')
         self.assertEqual(eth1.netmask, '255.255.252.0')
         self.assertEqual(eth1.broadcast, '1.12.123.255')
-        #        inet6 ffff::ffff:ffff:ffff:ffff  prefixlen 64  scopeid 0x20<link>
+        # inet6 ffff::ffff:ffff:ffff:ffff  prefixlen 64  scopeid 0x20<link>
         self.assertEqual(eth1.inet6, 'ffff::ffff:ffff:ffff:ffff')
         self.assertEqual(eth1.prefixlen, 64)
         self.assertEqual(eth1.scopeid, '0x20<link>')
@@ -651,34 +670,34 @@ vnet1: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500
         virbr0 = ifconfig.devices[2]
         self.assertEqual(virbr0.name, 'virbr0')
         # virbr0: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500
-        #         inet 1.12.123.124  netmask 255.255.252.0  broadcast 1.12.123.255
-        #         ether ff:00:11:22:33:42  txqueuelen 0  (Ethernet)
-        #         RX packets 88730  bytes 5140566 (4.9 MiB)
-        #         RX errors 0  dropped 0  overruns 0  frame 0
-        #         TX packets 177583  bytes 244647206 (233.3 MiB)
-        #         TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0
+        #  inet 1.12.123.124  netmask 255.255.252.0  broadcast 1.12.123.255
+        #  ether ff:00:11:22:33:42  txqueuelen 0  (Ethernet)
+        #  RX packets 88730  bytes 5140566 (4.9 MiB)
+        #  RX errors 0  dropped 0  overruns 0  frame 0
+        #  TX packets 177583  bytes 244647206 (233.3 MiB)
+        #  TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0
 
         # Verify vnet0:
         vnet0 = ifconfig.devices[3]
         self.assertEqual(vnet0.name, 'vnet0')
         # vnet0: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500
-        #         inet6 ffff::ffff:ffff:ffff:ffff  prefixlen 64  scopeid 0x20<link>
-        #         ether ff:00:11:22:33:42  txqueuelen 0  (Ethernet)
-        #         RX packets 538  bytes 172743 (168.6 KiB)
-        #         RX errors 0  dropped 0  overruns 0  frame 0
-        #         TX packets 26593  bytes 1379054 (1.3 MiB)
-        #         TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0
+        #  inet6 ffff::ffff:ffff:ffff:ffff  prefixlen 64  scopeid 0x20<link>
+        #  ether ff:00:11:22:33:42  txqueuelen 0  (Ethernet)
+        #  RX packets 538  bytes 172743 (168.6 KiB)
+        #  RX errors 0  dropped 0  overruns 0  frame 0
+        #  TX packets 26593  bytes 1379054 (1.3 MiB)
+        #  TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0
 
         # Verify vnet1:
         vnet1 = ifconfig.devices[4]
         self.assertEqual(vnet1.name, 'vnet1')
         # vnet1: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500
-        #         inet6 ffff::ffff:ffff:ffff:ffff  prefixlen 64  scopeid 0x20<link>
-        #         ether ff:00:11:22:33:42  txqueuelen 0  (Ethernet)
-        #         RX packets 71567  bytes 5033151 (4.7 MiB)
-        #         RX errors 0  dropped 0  overruns 0  frame 0
-        #         TX packets 216553  bytes 200424748 (191.1 MiB)
-        #         TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0
+        #  inet6 ffff::ffff:ffff:ffff:ffff  prefixlen 64  scopeid 0x20<link>
+        #  ether ff:00:11:22:33:42  txqueuelen 0  (Ethernet)
+        #  RX packets 71567  bytes 5033151 (4.7 MiB)
+        #  RX errors 0  dropped 0  overruns 0  frame 0
+        #  TX packets 216553  bytes 200424748 (191.1 MiB)
+        #  TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0
 
         fullOld = '''
 eth0      Link encap:Ethernet  HWaddr 00:11:22:33:44:55
@@ -707,23 +726,23 @@ lo        Link encap:Local Loopback
 
         # Verify eth0:
         eth0 = ifconfig.devices[0]
-        #eth0      Link encap:Ethernet  HWaddr 00:11:22:33:44:55
+        # eth0      Link encap:Ethernet  HWaddr 00:11:22:33:44:55
         self.assertEqual(eth0.name, 'eth0')
         self.assertEqual(eth0.hwtitle, 'Ethernet')
         self.assertEqual(eth0.hwaddr, '00:11:22:33:44:55')
-        #          inet addr:1.12.123.124  Bcast:1.12.123.255  Mask:255.255.252.0
+        # inet addr:1.12.123.124  Bcast:1.12.123.255  Mask:255.255.252.0
         self.assertEqual(eth0.inet, '1.12.123.124')
         self.assertEqual(eth0.netmask, '255.255.252.0')
         self.assertEqual(eth0.broadcast, '1.12.123.255')
-        #          inet6 addr: dddd::dddd:dddd:dddd:dddd:dddd:dddd/64 Scope:Site
-        #self.assertEqual(eth0.inet6, 'dddd::dddd:dddd:dddd:dddd:dddd:dddd')
-        #self.assertEqual(eth0.prefixlen, 64)
-        #self.assertEqual(eth0.scopeid, 'Site')
-        #          inet6 addr: eeee::eeee:eeee:eeee:eeee:eeee:eeee/64 Scope:Global
-        #self.assertEqual(eth0.inet6, 'eeee::eeee:eeee:eeee:eeee:eeee:eeee')
-        #self.assertEqual(eth0.prefixlen, 64)
-        #self.assertEqual(eth0.scopeid, 'Global')
-        #          inet6 addr: ffff::ffff:ffff:ffff:ffff/64 Scope:Link
+        #       inet6 addr: dddd::dddd:dddd:dddd:dddd:dddd:dddd/64 Scope:Site
+        # self.assertEqual(eth0.inet6, 'dddd::dddd:dddd:dddd:dddd:dddd:dddd')
+        # self.assertEqual(eth0.prefixlen, 64)
+        # self.assertEqual(eth0.scopeid, 'Site')
+        #       inet6 addr: eeee::eeee:eeee:eeee:eeee:eeee:eeee/64 Scope:Global
+        # self.assertEqual(eth0.inet6, 'eeee::eeee:eeee:eeee:eeee:eeee:eeee')
+        # self.assertEqual(eth0.prefixlen, 64)
+        # self.assertEqual(eth0.scopeid, 'Global')
+        #           inet6 addr: ffff::ffff:ffff:ffff:ffff/64 Scope:Link
         self.assertEqual(eth0.inet6, 'ffff::ffff:ffff:ffff:ffff')
         self.assertEqual(eth0.prefixlen, 64)
         self.assertEqual(eth0.scopeid, 'Link')
@@ -746,7 +765,7 @@ lo        Link encap:Local Loopback
         #          collisions:0 txqueuelen:1000
         self.assertEqual(eth0.txcollisions, 0)
         self.assertEqual(eth0.txqueuelen, 1000)
-        #          RX bytes:3380060233 (3.1 GiB)  TX bytes:713438255 (680.3 MiB)
+        # RX bytes:3380060233 (3.1 GiB)  TX bytes:713438255 (680.3 MiB)
         self.assertEqual(eth0.rxbytes, 3380060233)
         self.assertEqual(eth0.txbytes, 713438255)
         #          Interrupt:17 Memory:da000000-da012800
@@ -756,7 +775,7 @@ lo        Link encap:Local Loopback
 
         # Verify lo:
         lo = ifconfig.devices[1]
-        #lo        Link encap:Local Loopback
+        # lo        Link encap:Local Loopback
         self.assertEqual(lo.name, 'lo')
         self.assertEqual(lo.hwtitle, 'Local Loopback')
         self.assertEqual(lo.hwaddr, None)
@@ -777,7 +796,7 @@ lo        Link encap:Local Loopback
         #          TX packets:805234 errors:0 dropped:0 overruns:0 carrier:0
         self.assertEqual(lo.txpackets, 805234)
         #          collisions:0 txqueuelen:0
-        #          RX bytes:132177529 (126.0 MiB)  TX bytes:132177529 (126.0 MiB)
+        # RX bytes:132177529 (126.0 MiB)  TX bytes:132177529 (126.0 MiB)
         self.assertEqual(lo.rxbytes, 132177529)
         self.assertEqual(lo.txbytes, 132177529)
 
@@ -795,7 +814,8 @@ lo        Link encap:Local Loopback
         self.assertEqual(dev.mtu, 1500)
 
         dev = self.parse_single_device(
-            'foo: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500  outfill 32  keepalive 96\n')
+            'foo: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500  '
+            'outfill 32  keepalive 96\n')
         self.assertEqual(dev.mtu, 1500)
         self.assertEqual(dev.outfill, 32)
         self.assertEqual(dev.keepalive, 96)
@@ -803,7 +823,8 @@ lo        Link encap:Local Loopback
         # old format
         dev = self.parse_single_device(
             'foo      Link encap:Ethernet  HWaddr F0:F0:F0:F0:F0:F0\n'
-            '         UP BROADCAST RUNNING MULTICAST  MTU:500  Metric:2  Outfill:55  Keepalive:66\n')
+            '         UP BROADCAST RUNNING MULTICAST  MTU:500  Metric:2  '
+            'Outfill:55  Keepalive:66\n')
         self.assertEqual(dev.flagsstr, 'UP BROADCAST RUNNING MULTICAST')
         self.assertEqual(dev.mtu, 500)
         self.assertEqual(dev.metric, 2)
@@ -813,7 +834,8 @@ lo        Link encap:Local Loopback
     def test_parsing_ip(self):
         dev = self.parse_single_device(
             'foo: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500\n'
-            '        inet 1.12.123.124  netmask 255.255.252.0  broadcast 1.12.123.255\n')
+            '        inet 1.12.123.124  netmask 255.255.252.0  broadcast '
+            '1.12.123.255\n')
         self.assertEqual(dev.inet, '1.12.123.124')
         self.assertEqual(dev.netmask, '255.255.252.0')
         self.assertEqual(dev.get_netmask_bits(), 22)
@@ -832,7 +854,8 @@ lo        Link encap:Local Loopback
 
         dev = self.parse_single_device(
             'foo: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500\n'
-            '        inet 1.1.1.1  netmask 255.255.255.254  destination 1.12.123.255\n')
+            '        inet 1.1.1.1  netmask 255.255.255.254  destination '
+            '1.12.123.255\n')
         self.assertEqual(dev.inet, '1.1.1.1')
         self.assertEqual(dev.netmask, '255.255.255.254')
         self.assertEqual(dev.broadcast, None)
@@ -841,7 +864,8 @@ lo        Link encap:Local Loopback
         # old format
         dev = self.parse_single_device(
             'foo      Link encap:Ethernet  HWaddr F0:F0:F0:F0:F0:F0\n'
-            '       inet addr:1.2.3.4  P-t-P:10.20.30.40  Mask:255.255.254.0\n')
+            '       inet addr:1.2.3.4  P-t-P:10.20.30.40  '
+            'Mask:255.255.254.0\n')
         self.assertEqual(dev.inet, '1.2.3.4')
         self.assertEqual(dev.destination, '10.20.30.40')
         self.assertEqual(dev.broadcast, None)
@@ -908,12 +932,13 @@ lo        Link encap:Local Loopback
         self.assertEqual(dev.txcompressed, 11)
 
     def test_parse_ip4addr(self):
-        self.assertEqual(parse_ip4addr('1.1.1.1'), '\x01\x01\x01\x01')
-        self.assertEqual(parse_ip4addr('127.0.0.1'), '\x7f\x00\x00\x01')
+        self.assertEqual(parse_ip4addr('1.1.1.1'), b'\x01\x01\x01\x01')
+        self.assertEqual(parse_ip4addr('127.0.0.1'), b'\x7f\x00\x00\x01')
 
     def test_local(self):
         # Actually invoke ifconfig locally, and parse whatever it emits:
-        ifconfig = IfConfig()
+        IfConfig()
+
 
 if __name__ == '__main__':
     unittest.main()
